(*
 *
 * Copyright (C) 2003 Damien Pous
 * 
 * OCaml-lirc is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OCaml-lirc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OCaml-lirc; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)


open Sys


(* exception gnrique du module *)
exception Lirc of string

(* exception raised when the LIRC daemon is shut down *)
exception DaemonDown

(* type abstrait pour la config *)
type t


(* interfaage avec la librairie C *)
(* ne sachant pas utiliser les rfrences caml en C, je renvoie des couples *)
(* la valeur n'a un sens que si l'entier est  0 (pas d'erreur)...          *)
external _lirc_init       : string ->        (* prog        *)
                           int           = "ocalirc_init"

external _lirc_deinit     : unit   ->
                           int           = "ocalirc_deinit"
 
external _lirc_readconfig : string ->        (* file   *)
                           string  ->        (* check  *) 
                           int*t         = "ocalirc_readconfig"

external _lirc_freeconfig : t ->             (* config *)
                           unit          = "ocalirc_freeconfig"

external _lirc_nextcode   : unit ->         
                           int*string    = "ocalirc_nextcode"

external _lirc_code2char  : t     ->         (* config *)
                           string ->         (* code   *)
			   int*string    = "ocalirc_code2char"


(* initialisation du client LIRC *)
let init prog = 
  match _lirc_init prog with
  | -1 -> raise (Lirc "init")
  | _ -> ()


(* libration du client LIRC *)
let deinit() =
  match _lirc_deinit() with
  | 0 -> ()
  | _ -> raise (Lirc "deinit")


(* lecture du fichier de configuration *)
let readconfig ?(check=(fun _->true)) ?(config="") () = 
  let check_s = "lirc_check_function" in
    Callback.register check_s (fun s -> if check s then 0 else -1);
    match _lirc_readconfig config check_s with
      | 0, config -> config
      | _ -> raise (Lirc "readconfig")
           


(* libration du fichier de la valeur "config" obtenue prcdemment *)
let freeconfig = _lirc_freeconfig


(* obtention des chanes correspondant au dernier code reu *)
let nextcode config = 
  let rec aux c =
    match _lirc_code2char config c with
    | 0,s -> s::(aux c)
    | 51,_ -> []
    | _ -> raise DaemonDown
  in
  match _lirc_nextcode() with
  | 0,code -> aux code
  | 51,_ -> [] 
  | _ -> raise DaemonDown




(* fonction de haut niveau *)
let connect prog ?(config="") ?(check=(fun _->true)) f = 

(* initialisation de la connection au dmon lircd *)
  init prog; 

  try

(* lecture du fichier de configuratiopn *)
    let c = readconfig ~check ~config () in 

(* fonction rcursive terminale, travaillant sur *)
(* un liste ravitaille lorsqu'elle est vide...  *)
    let rec aux = function
      | [] -> aux (nextcode c)
      | x::q ->
	  if f x then
	    aux q
	  else (
	    freeconfig c;
	    deinit()
	  )
    in

    try
      aux []

    with
      DaemonDown ->
	freeconfig c;
	raise DaemonDown   (* on propage *)

  with
  | e -> deinit(); raise e (* on propage aussi *)
