(*
 *
 * Copyright (C) 2003 Damien Pous
 * 
 * OCaml-lirc is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OCaml-lirc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OCaml-lirc; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)


(**

    This module provides access to {{:http://www.lirc.org/}LIRC}    
    (see also the project {{:http://perso.ens-lyon.fr/damien.pous/shared/ocaml/lirc}homepage})
    
*)

(** generic exception for the module *)
exception Lirc of string

(** exception raised when the LIRC daemon is shut down *)
exception DaemonDown





(* ------------------------ *)
(** {1 High level function} *)
(* ------------------------ *)

(** [connect prog handler] initialize a client with name [prog]
  then call [handler] each time a code is received, until it return [false]
  @param check allow to check each code found in the configuration file (defaults to the constant true function) 
  @param config path to the configuration file (defaults to {i ~/.lircrc})
*)
 val connect :
      string
   -> ?config:string 
   -> ?check: (string -> bool) 
   -> (string -> bool)
   -> unit
   



(* ------------------------ *)
(** {1 Low level functions} *)
(* ------------------------ *)


(** configuration file handle *)
type t


(** [init prog] initialize the lirc client with name [prog] *)
val init : string -> unit

(** lirc client release *)
val deinit : unit -> unit


(** reading of the configuration file
  @param check allow to check each code found in the configuration file (defaults to the constant true function) 
  @param config path to the configuration file (defaults to {i ~/.lircrc})
  @raise Lirc"readconfig" if [config] does not exists or [check] return false.
*)
val readconfig : ?check:(string -> bool) -> ?config:string -> unit -> t


(** release of the configuration handle *)
val freeconfig : t -> unit


(** [nextcode t] return the list of the codes received 
  since the last call. If no codes were received, wait for a code. *)
val nextcode : t -> string list

