/*
 *
 * Copyright (C) 2003 Damien Pous
 * 
 * OCaml-lirc is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OCaml-lirc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OCaml-lirc; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */


#include <lirc/lirc_client.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/callback.h>
#include <assert.h>

//#define DEBUG

#ifdef DEBUG
#include <stdio.h>
#include <stdlib.h>
#define DBG(args...) fprintf(stderr, args)
#else
#define DBG(args...)
#endif

static value *_ocalirc_check_caml = NULL;
int _ocalirc_check(char *s)
{
  assert(_ocalirc_check_caml != NULL); 
  return Int_val(callback(*_ocalirc_check_caml, copy_string(s)));
}

value ocalirc_init(value prog)
{
  CAMLparam1 (prog);
  DBG("init : '%s'\n", String_val(prog))
  CAMLreturn (Val_int(lirc_init(String_val(prog), 0)));
}

value ocalirc_deinit(value unit)
{
  CAMLparam1 (unit);
  DBG("deinit !\n");
  CAMLreturn (Val_int(lirc_deinit()));
}

value ocalirc_readconfig(value file, value check)
{
  CAMLparam2 (file, check);
  CAMLlocal1 (res);

  struct lirc_config *config = NULL;
  int err;
  char *f;
  int (*check_f)(char *) = NULL;

  if (*String_val(check) != 0)  
    {
      _ocalirc_check_caml = caml_named_value(String_val(check));
      check_f = _ocalirc_check;
    }

  f=String_val(file);
  if (*f==0) f=NULL;
  DBG("readconfig : '%s'\n", f ? f : "NULL" )

  res = alloc (2, 0);
  if ((err=lirc_readconfig(f, &config, check_f))==0)
    Store_field (res, 1, (value) config);      
  Store_field (res, 0, Val_int(err));
  DBG("returned (%i, %p)\n", err, config)
  CAMLreturn (res);
}

value ocalirc_freeconfig(value config)
{
  CAMLparam1 (config);
  DBG("freeconfig : %p\n", (struct lirc_config *)config)
  lirc_freeconfig((struct lirc_config *)config);
  CAMLreturn (Val_unit);
}

value ocalirc_nextcode (value unit)
{
  CAMLparam1 (unit);
  CAMLlocal1 (res);
  int err;
  char *code;
  DBG("nextcode\n")
  res = alloc (2, 0);
  if ((err=lirc_nextcode(&code))==0)
    {
      if (code)
	{
	  Store_field (res, 1, copy_string(code));
	  DBG("returned (%i, '%s')\n", err, code)
	  free(code);
	}
      else
	{
	  err=51;
	  DBG("returned (51, 'NULL')\n")
	}
    }
  else {DBG("nextcode : error\n")}

  Store_field (res, 0, Val_int(err));
  CAMLreturn (res);
}

/* value ocalirc_freecode (value code) */
/* { */
/*   CAMLparam1(code); */
/*   DBG("freecode : '%s'\n", String_val(code)) */
/*   free(String_val(code)); */
/*   CAMLreturn(Val_unit); */
/* } */

value ocalirc_code2char (value config, value code)
{
  CAMLparam2 (config, code);
  CAMLlocal1 (res);
  char *string;
  int err;
  DBG("code2char : %p '%s'\n", (struct lirc_config *)config, String_val(code))
  res = alloc (2, 0);
  if ((err=lirc_code2char((struct lirc_config *)config, String_val(code), &string))==0)
    {
      if (string)
	{
	  Store_field (res, 1, copy_string(string));      
	    //	  free(string);  // !??????
	}
      else
	err=51;
    }
  DBG("returned (%i, '%s')\n", err, string)
  Store_field (res, 0, Val_int(err));
  CAMLreturn (res);
}













